{
    Copyright (c) 2020 Andreas Friess

    Licence LGPL with extension

    See the file COPYING.modified.LGPL for details about the copyright.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

 **********************************************************************}
unit ImgButton;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, ExtCtrls;

type

  { TImgButton }

  TImgButton = class(TCustomImage)
  private
    FBorder: integer;
    FColor: TColor;
    FImageIndex: integer;
    FImageIndexClick: integer;
    FImageIndexHover: integer;
    BuState: integer;
    FImageList: TImageList;
    FImageListClick: TImageList;
    FImageListHover: TImageList;
    procedure SetImage;
  protected
    procedure Paint; override;
    procedure Click; override;
    procedure MouseDown(Button: TMouseButton; Shift:TShiftState; X,Y:Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift:TShiftState; X,Y:Integer); override;
    procedure MouseEnter; override;
    procedure MouseLeave; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Border: integer read FBorder write FBorder default 3;
    property Color: TColor read FColor write FColor default clDefault;
    property ImageIndexClick: integer read FImageIndexClick write FImageIndexClick default -1;
    property ImageListClick: TImageList read FImageListClick write FImageListClick default nil;
    property ImageIndexHover: integer read FImageIndexHover write FImageIndexHover default -1;
    property ImageListHover: TImageList read FImageListHover write FImageListHover default nil;
    property ImageIndex: integer read FImageIndex write FImageIndex default -1;
    property ImageList: TImageList read FImageList write FImageList default nil;
    // stolen from TImage in ExtCtrl
    property AntialiasingMode;
    property Align;
    property Anchors;
    property AutoSize;
    property BorderSpacing;
    property Center;
    property KeepOriginXWhenClipped;
    property KeepOriginYWhenClipped;
    property Constraints;
    property DragCursor;
    property DragMode;
    property Enabled;
    property OnChangeBounds;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseWheel;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    property OnPaint;
    property OnPictureChanged;
    property OnPaintBackground;
    property OnResize;
    property OnStartDrag;
    property ParentShowHint;
    property Picture;
    property PopupMenu;
    property Proportional;
    property ShowHint;
    property Stretch;
    property StretchOutEnabled;
    property StretchInEnabled;
    property Transparent;
    property Visible;
  end;

procedure Register;

implementation
uses
  Types;

procedure Register;
begin
  {$I imgbutton_icon.lrs}
  RegisterComponents('AF',[TImgButton]);
end;

const
  buInvalid = -1;           // for startup/reset
  buNothing = 0;            // normal state
  buHover = 1;              // mouse is over image
  buClicked = 2;            // is clicked

{ TImgButton }

procedure TImgButton.Click;
begin
  inherited Click;
  BuState:= buClicked;
  SetImage;
end;

procedure TImgButton.MouseDown(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
begin
  inherited MouseDown(Button, Shift, X, Y);
  BuState:= buClicked;
  SetImage;
end;

procedure TImgButton.MouseUp(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
begin
  inherited MouseUp(Button, Shift, X, Y);
  if PtInRect(self.BoundsRect, ScreenToClient(Mouse.CursorPos)) then
     BuState:= buHover
   else
     BuState:= buNothing;
  SetImage;
end;

procedure TImgButton.MouseEnter;
begin
  inherited MouseEnter;
  BuState:= buHover;
  SetImage;
end;

procedure TImgButton.MouseLeave;
begin
  inherited MouseLeave;
  BuState:=buNothing;
  SetImage;
end;

procedure TImgButton.SetImage;
var
  MyBmp: TBitmap;
begin
  MyBmp:= TBitmap.Create;
  try
    case BuState of
      buHover:
        if (ImageListHover <> nil) and (ImageIndexHover >= 0) then begin
          if (ImageIndexHover < ImageListHover.Count) then begin
            ImageListHover.GetBitmap(ImageIndexHover,MyBmp);
          end;
        end;
      buClicked:
        if (ImageListClick <> nil) and (ImageIndexClick >= 0) then begin
          if (ImageIndexClick < ImageListClick.Count) then begin
            ImageListClick.GetBitmap(ImageIndexClick,MyBmp);
          end;
        end;
    else
      if (ImageList <> nil) and (ImageIndex >= 0) then begin
        if (ImageIndex < ImageList.Count) then begin
          ImageList.GetBitmap(ImageIndex,MyBmp);
        end;
      end;
    end;
    Picture.Bitmap:= MyBmp;
  finally
    MyBmp.Free;
  end;
end;

procedure TImgButton.Paint;
begin
  if (BuState = buInvalid) then begin
    BuState:=buNothing;
    SetImage;
  end;
  inherited Paint;
end;


constructor TImgButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  // for startup
  // because at startup the ImageLists can be empty, if the
  // reader have not loaded the content before
  // we must delay the first load
  BuState:= buInvalid;
end;

end.
